const cheerio = require('cheerio');
const fetch = require('node-fetch');
const ora = require('ora');
const klaw = require('klaw');

exports.DEFAULT_INFO = {
  site: '',
  id: '',
  begin: '',
  official: null,
  premuiumOnly: null,
  censored: null,
  exist: null,
  comment: '',
};

/**
 * 合并两个番剧对象数组
 * @param  {Array}  targets            目标数组
 * @param  {Array}  sources            源数组
 * @param  {String}  identifier        对象识别值
 * @param  {Boolean} [overwrite=false] 是否覆写目标对象
 * @return {Array}
 */
exports.merge = function merge(targets, sources, identifier, overwrite = false) {
  sources.forEach((source) => {
    const target = targets.find(t => (
      identifier === null ? t === source : t[identifier] === source[identifier]
    ));
    if (!target) {
      targets.push(source);
      return;
    }
    for (const key in source) {
      if (key === 'titleTranslate') {
        for (const lang in source[key]) {
          target[key][lang] = merge(
            target[key][lang] || [],
            source[key][lang],
            null,
            overwrite,
          );
        }
      } else if (key === 'sites') {
        target[key] = merge(target[key], source[key], 'site', overwrite);
      } else {
        if (
          source[key] === ''
          || source[key] === null
          || (target[key] !== '' && target[key] !== null && !overwrite)
        ) {
          continue;
        }
        target[key] = source[key];
      }
    }
  });
  return targets;
};

/**
 * 将番剧对象数组转换为按年月分类的对象
 * @param  {Array} items 番剧数组
 * @return {Object}
 */
exports.classify = function classify(items) {
  const data = {};
  items.forEach((item) => {
    // 对于无放送开始时间的番组，将其保存到 `./0000/00.json` 手动处理
    const begin = item.begin || '0000-00';
    const year = begin.slice(0, 4);
    const month = begin.slice(5, 7);
    data[year] = data[year] || {};
    data[year][month] = data[year][month] || [];
    data[year][month].push(item);
  });
  return data;
};

exports.delay = function delay(time) {
  return new Promise((resolve) => {
    setTimeout(resolve, time);
  });
};

exports.timeout = function timeout(time) {
  return new Promise((resolve, reject) => {
    setTimeout(() => reject(new Error('operation timed out')), time);
  });
};

exports.getAllBangumi = async function getAllBangumi(gen, page = 1, total = 0) {
  const spinner = ora(
    gen.message
      ? gen.message(page, total)
      : `Crawling page ${page}/${total || '?'}`,
  ).start();
  const api = gen.api(page);
  const $ = await fetch(api)
    .then(res => res.text())
    .then(cheerio.load);
  spinner.stop();
  const totalPage = gen.total($, page);
  const items = gen.items($);
  if (page < totalPage) {
    return items.concat(await getAllBangumi(gen, page + 1, totalPage));
  }
  return items;
};

exports.walk = function walk(dir, filter) {
  return new Promise((resolve, reject) => {
    const items = [];
    klaw(dir)
      .on('data', (item) => {
        if (filter(item.path)) {
          items.push(item.path);
        }
      })
      .on('end', () => {
        resolve(items);
      })
      .on('error', reject);
  });
};
